// was used with webpack configuration
// kept for public API reference

const mashupConfig = require('mashup-config');

const dependencyNames = mashupConfig.dependencies.map(
  dependency =>
    typeof dependency === 'string' ? dependency : dependency.mashupDependency
);

let initializedDependencyMap = null;

function initializeReplacedModules(dependencies) {
  if (!initializedDependencyMap) {
    initializedDependencyMap = {};
    dependencyNames.forEach((name, index) => {
      initializedDependencyMap[name] = dependencies[index];
    });

    replacedModules = {};

    mashupConfig.dependencies.forEach(dependency => {
      if (
        typeof dependency !== 'string' &&
        dependency.provideModule != undefined
      ) {
        const {provideModule, mashupDependency} = dependency;
        replacedModules[provideModule] =
          initializedDependencyMap[mashupDependency];
      }
    });
  }

  return initializedDependencyMap;
}

function addDependencies(mashup) {
  return dependencyNames.reduce(
    (m, dependency) => m.addDependency(dependency),
    mashup
  );
}

function addModule(mashup, name, moduleFunction) {
  addDependencies(mashup);

  mashup.addModule(name, function() {
    const dependencies = Array.prototype.slice.call(arguments);
    const dependencyMap = initializeReplacedModules(dependencies);
    return moduleFunction(dependencyMap);
  });

  return mashup;
}

function addMashup(mashup, mashupFunction) {
  addDependencies(mashup);

  mashup.addMashup(function() {
    const dependencies = Array.prototype.slice.call(arguments);
    const dependencyMap = initializeReplacedModules(dependencies);
    const {initialize} = mashupFunction(dependencyMap);
    initialize();
  });

  return mashup;
}

function registerPublicApi(mashup) {
  if (mashupConfig.publicApi) {
    const apiModule = require('mashup-public-api');
    const apis =
      apiModule.__esModule && apiModule.default ? apiModule.default : apiModule;
    for (const name in apis) {
      if (apis.hasOwnProperty(name)) {
        addModule(mashup, name, apis[name]);
      }
    }
  }

  return mashup;
}

registerPublicApi(tau.mashups);

const initializerFunction = dependencyMap => {
  // module needs to be loaded asynchronously because dependencies
  // should only be resolved AFTER replacedModules are defined
  const {initialize} = require('mashup-entry-point');

  return {
    initialize: (...args) => initialize(dependencyMap, ...args)
  };
};

if (process.env.target === 'mashup') {
  addMashup(tau.mashups, initializerFunction);
} else {
  addModule(tau.mashups, mashupConfig.moduleName, initializerFunction);
}
