const createRollupConfig = require('../../src/config/rollup.config.default');
const path = require('path');

const fileTransformFunctionMock = jest.fn(x => x);
const configTransformFileMock = jest.mock(
  path.join(__dirname, './from-file/configTransform.js'),
  () => fileTransformFunctionMock,
  {virtual: true}
);
const configTransformFunctionMock = jest.fn(x => x);
const mashupConfigJsMock = jest.mock(
  path.join(__dirname, './from-function/mashup.config.js'),
  () => ({
    name: 'mock-mashup-config',
    configTransform: configTransformFunctionMock
  })
);
const postcssConfigFunctionMock = jest.fn(() => {
  plugins: [];
});
const postcssConfigMock = jest.mock(
  path.join(__dirname, './postcss/postcss.config.js'),
  postcssConfigFunctionMock
);

describe('rollup config factory', () => {
  it('should default to /src/index.js as entry point', async () => {
    const config = await createRollupConfig({
      dir: path.join(__dirname, 'default-entry')
    });

    expect(config.input[0]).toBe(
      path.join(__dirname, './default-entry', './src/index.js')
    );
  });

  it('should use "entry", if specified, as entry point', async () => {
    const config = await createRollupConfig({
      dir: path.join(__dirname, 'custom-entry')
    });

    expect(config.input[0]).toBe(
      path.join(__dirname, './custom-entry', './mashup.js')
    );
  });

  it('should obey config transforms when transform file path is specified', async () => {
    const config = await createRollupConfig({
      dir: path.join(__dirname, 'from-file')
    });

    expect(fileTransformFunctionMock).toHaveBeenCalledWith(config);
  });

  it('should obey config transforms when transform is passed as a function', async () => {
    const config = await createRollupConfig({
      dir: path.join(__dirname, 'from-function')
    });

    expect(configTransformFunctionMock).toHaveBeenCalledWith(config);
  });

  it('should autodetect and use postcss config provided with mashup', async () => {
    const config = await createRollupConfig({
      dir: path.join(__dirname, 'postcss')
    });

    expect(postcssConfigFunctionMock).toHaveBeenCalled();
  });
});
